/* The following code declares a base class for the class array,
 * an STL container (as wrapper) for arrays of constant size.
 *
 * See
 *      http://www.boost.org/libs/array/
 * for documentation.
 *
 * The original author site is at: http://www.josuttis.com/
 *
 * (C) Copyright Nicolai M. Josuttis 2001.
 * (C) Copyright Peter Myerscough-Jackopson, 2010.
 *
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 *
 * 15 Mar 2010 - enabled a base class to allow pointers to arrays to contain different sized arrays 
 * 29 Jan 2004 - c_array() added, BOOST_NO_PRIVATE_IN_AGGREGATE removed (Nico Josuttis)
 * 23 Aug 2002 - fix for Non-MSVC compilers combined with MSVC libraries.
 * 05 Aug 2001 - minor update (Nico Josuttis)
 * 20 Jan 2001 - STLport fix (Beman Dawes)
 * 29 Sep 2000 - Initial Revision (Nico Josuttis)
 *
 * Mar 15, 2010
 */
#ifndef BOOST_ARRAY_BASE_HPP
#define BOOST_ARRAY_BASE_HPP

#include <boost/detail/workaround.hpp>

#if BOOST_WORKAROUND(BOOST_MSVC, >= 1400)  
# pragma warning(push)  
# pragma warning(disable:4996) // 'std::equal': Function call with parameters that may be unsafe
#endif

#include <cstddef>
#include <stdexcept>
#include <boost/assert.hpp>
#include <boost/swap.hpp>

// Handles broken standard libraries better than <iterator>
#include <boost/detail/iterator.hpp>
#include <boost/throw_exception.hpp>
#include <algorithm>

// FIXES for broken compilers
#include <boost/config.hpp>

namespace boost {

    template<class T>
    class array_base {

      public:
        // type definitions
        typedef T              value_type;
        typedef T*             iterator;
        typedef const T*       const_iterator;
        typedef T&             reference;
        typedef const T&       const_reference;
        typedef std::size_t    size_type;
        typedef std::ptrdiff_t difference_type;

        // iterator support
        virtual iterator begin() = 0;
        virtual const_iterator begin() const = 0;
        virtual iterator end() = 0;
        virtual const_iterator end() const = 0;

        // reverse iterator support
#if !defined(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION) && !defined(BOOST_MSVC_STD_ITERATOR) && !defined(BOOST_NO_STD_ITERATOR_TRAITS)
        typedef std::reverse_iterator<iterator> reverse_iterator;
        typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
#elif defined(_MSC_VER) && (_MSC_VER == 1300) && defined(BOOST_DINKUMWARE_STDLIB) && (BOOST_DINKUMWARE_STDLIB == 310)
        // workaround for broken reverse_iterator in VC7
        typedef std::reverse_iterator<std::_Ptrit<value_type, difference_type, iterator,
                                      reference, iterator, reference> > reverse_iterator;
        typedef std::reverse_iterator<std::_Ptrit<value_type, difference_type, const_iterator,
                                      const_reference, iterator, reference> > const_reverse_iterator;
#else
        // workaround for broken reverse_iterator implementations
        typedef std::reverse_iterator<iterator,T> reverse_iterator;
        typedef std::reverse_iterator<const_iterator,T> const_reverse_iterator;
#endif

        virtual reverse_iterator rbegin() = 0;
        virtual const_reverse_iterator rbegin() const  = 0;
        virtual reverse_iterator rend() = 0;
        virtual const_reverse_iterator rend() const = 0;

        // operator[]
        virtual reference operator[](size_type i) = 0;
        
        virtual const_reference operator[](size_type i) const = 0;

        // at() with range check
        virtual reference at(size_type i) = 0;
        virtual const_reference at(size_type i) const = 0;
    
        // front() and back()
        virtual reference front() = 0;
        virtual const_reference front() const = 0;
        virtual reference back() = 0;
        virtual const_reference back() const = 0;

        // size is constant
        virtual size_type size() const = 0;
        virtual bool empty() const = 0;
        virtual size_type max_size() const = 0;
        //enum { static_size = size() };

        // swap (note: linear complexity)
        // virtual void swap (array_base<T>& y) = 0;

        // direct access to data (read-only)
        virtual const T* data() const = 0;
        virtual T* data() = 0;

        // use array as C array (direct read/write access to data)
        virtual T* c_array() = 0;

        // assignment with type conversion
        //template <typename T2>
        //virtual array_base<T>& operator= (const array_base<T2>& rhs) = 0;

        // assign one value to all elements
        virtual void assign (const T& value) = 0;

        // check range 
        virtual void rangecheck (size_type i) const = 0;
    };
} /* namespace boost */


#if BOOST_WORKAROUND(BOOST_MSVC, >= 1400)  
# pragma warning(pop)  
#endif 


#endif /*BOOST_ARRAY_BASE_HPP*/
